//+------------------------------------------------------------------+
//|                                                       Candle.mqh |
//|                                              Copyright 2010, Kos |
//|                                      http://mql5.com/ru/users/kos|
//+------------------------------------------------------------------+
#property copyright "2010, Kos"

#include <JCPE\Var.mqh>

int BearLongBottomShadows[15] = {18,19,22,23,27,34,35,38,39,43,50,51,54,55,59};
int BullLongBottomShadows[15] = {82,83,86,87,91,98,99,102,103,107,114,115,118,119,123};
int BearLongTopShadows[15]    = {24,28,25,29,30,40,41,44,45,46,56,57,60,61,62};
int BullLongTopShadows[15]    = {88,89,92,93,94,104,105,110,108,109,120,121,124,125,126};
int HV[12]                    = {85,90,95,21,26,31,106,111,42,47,127,63};
int MB[16]                    = {96,97,100,112,113,116,32,33,36,48,49,52,16,17,20,21};
//+------------------------------------------------------------------+
//|                                                                  |
//+------------------------------------------------------------------+
class CCandle
  {
protected:
   void              GetCandleCode();
   uchar             CodeGenerate(int Bar);
   bool              CandleWithLongTopShadow(int Bar,uchar TypeModel);
   bool              CandleWithLongBottomShadow(int Bar,uchar TypeModel);
   MqlRates          GlobalRates[];
   uchar             Codes[];
   int               tplSizes[9];   //    (,)  3     -  3   :   ,  ,  
   CLoaderManager   *LM;
public:
                    ~CCandle(void);
   int               SetRates(ENUM_TIMEFRAMES TF,string SName,int History_point);
   void              SetEngineSettings();
   bool              LongTopShadows(int Bar);
   bool              LongBottomShadows(int Bar);
   double            Op(int Bar);
   double            Hi(int Bar);
   double            Lo(int Bar);
   double            Cl(int Bar);
   double            Mid(int Bar);
   uchar             Code(int Bar);
   int               BodySize(int Bar);
   int               TopShadowSize(int Bar);
   int               BottomShadowSize(int Bar);
   uchar             Type(int Bar);
   bool              LongCandle(int Bar,double deviation=1);
   bool              MidlleCandle(int Bar);
   bool              SmallCandle(int Bar);
   bool              Doji(int Bar);
   bool              BearBar(int Bar);
   bool              BullBar(int Bar);
   bool              DragonDojiCandle(int Bar);
   bool              GraveStoneDojiCandle(int Bar);
   bool              LongLegsDojiCandle(int Bar);
   bool              Maribozu(int Bar);
   ushort            CheckMaribozu(int Bar);
   bool              HighWave(int Bar);
   ushort            HangingManCandle(int Bar);
   ushort            HummerCandle(int Bar);
   ushort            InvertedHummerCandle(int Bar);
   ushort            ShootingStarCandle(int Bar);
   bool              BearPinBar(int Bar);
   bool              BullPinBar(int Bar);
   void              CheckData();
  };
//+------------------------------------------------------------------+
//|                                                                  |
//+------------------------------------------------------------------+
void CCandle::~CCandle(void)
  {
   if(CheckPointer(LM)!=POINTER_INVALID)
     {
      if(CheckPointer(LM)==POINTER_DYNAMIC) delete(LM);
     }
  }
//+------------------------------------------------------------------+
//|                                                                  |
//+------------------------------------------------------------------+
int  CCandle::SetRates(ENUM_TIMEFRAMES TF,string SName,int History_point)
  {
   LM=new CLoaderManager;
   int res=5;
   if(LM.InitBodyElements(SName,TF,tplSizes,res))
     {
      bool Continue=false;
      switch(res)
        {
         case -1:
         case 0:
           {
            Continue=(LM.SizesArrayLoad(SName,TF,tplSizes)>0);
           }
         break;

         case 1:
           {
            Continue=true;
           }
         break;
        }

      if(Continue)
        {
         int Count=CopyRates(Name,TF,History_point,SHIFT,GlobalRates);
         if(Count>0)
           {
            pips=MathPow(10,digits);
            ArraySetAsSeries(GlobalRates,true);
            return(Count);
           }
        }
     }
   delete LM;
   LM=NULL;
   return(0);
  }
//    

double CCandle::Op(int Bar)
  {
   return(NormalizeDouble(GlobalRates[Bar].open,digits));
  }
//   

double CCandle::Hi(int Bar)
  {
   return(NormalizeDouble(GlobalRates[Bar].high,digits));
  }
//   

double CCandle::Lo(int Bar)
  {
   return(NormalizeDouble(GlobalRates[Bar].low,digits));
  }
//   

double CCandle::Cl(int Bar)
  {
   return(NormalizeDouble(GlobalRates[Bar].close,digits));
  }
//     

double CCandle::Mid(int Bar)
  {
   double half=MathCeil(BodySize(Bar)/2);
   double res,top;
   if(Type(Bar)!=DOJICANDLE)
     {
      top=MathMax(Op(Bar),Cl(Bar));
      res=top-half*point;
     }
   else res=Cl(Bar);
   return(NormalizeDouble(res,digits));
  }
//  

uchar CCandle::Code(int Bar)
  {
   return(Codes[Bar]);
  }
//  

void CCandle::SetEngineSettings()
  {
   int Template=tplSizes[SHORT_CANDLE_BODY];
   GGap          =MathCeil(Template*Settings.ES_GAP);
   GDoji         =MathCeil(Template*Settings.ES_Doji);
   G_Error       =MathCeil(Template*Settings.ES_Error)/pips;
   ArrayInitialize(Codes,0);
   ArrayResize(Codes,CODE_SHIFT);
   ArraySetAsSeries(Codes,true);
   GetCandleCode();
  }
//  

void CCandle::GetCandleCode()
  {
   uint i;
   for(i=Custom.Low(Codes); i<=Custom.High(Codes);i++)
     {
      Codes[i]=CodeGenerate(i);
     }
  }
//  

uchar  CCandle::CodeGenerate(int Bar)
  {
   uchar    code=0;
   uchar    Step=0,metod=0;
   int    body,ushd,lshd;

   body=BodySize(Bar);
   ushd=TopShadowSize(Bar);
   lshd=BottomShadowSize(Bar);

   switch(Type(Bar))
     {
      case BULLCANDLE:
         Step  =64;
         metod =0;
         break;

      case BEARCANDLE:
         Step  =0;
         metod =0;
         break;

      case DOJICANDLE:
         Step  =64;
         metod =1;
         break;
     }

   if(metod==0)
     {
      if(body>tplSizes[LONG_CANDLE_BODY]) code+=48;
      else
         if(body>tplSizes[SHORT_CANDLE_BODY]) code+=32;
      else                                                            code+=16;
     }

   if(ushd>=tplSizes[LONG_TOP_SHADOW]) code+=12;
   else
      if((ushd>tplSizes[SHORT_TOP_SHADOW]) && 
         (ushd<tplSizes[LONG_TOP_SHADOW])) code+=8;
   else
      if((ushd<=tplSizes[SHORT_TOP_SHADOW]) && 
         (ushd>GDoji)) code+=4;
   else                                                      code+=0;

   if(lshd>=tplSizes[LONG_BOTTOM_SHADOW]) code+=3;
   else
      if((lshd>tplSizes[SHORT_BOTTOM_SHADOW]) && 
         (lshd<tplSizes[LONG_BOTTOM_SHADOW])) code+=2;
   else
      if((lshd<=tplSizes[SHORT_BOTTOM_SHADOW]) && 
         (lshd>GDoji)) code+=1;
   else                                                       code+=0;

   return(code+Step);
  }
//+------------------------------------------------------------------+
//|                                                                  |
//+------------------------------------------------------------------+
void CCandle::CheckData(void)
  {
   Print("Code:",Codes[1],",",Codes[2],",",Codes[3],",",Codes[4],",",Codes[5]);
  }
//   

int CCandle::BodySize(int Bar)
  {
   int res=0;
   if(GlobalRates[Bar].open==GlobalRates[Bar].close) return(1);
   else
     {
      res=int(MathAbs(GlobalRates[Bar].open-GlobalRates[Bar].close)*pips);
      return(res);
     }
  }
//    

int CCandle::TopShadowSize(int Bar)
  {
   int res=0;
   res=int((GlobalRates[Bar].high-MathMax(GlobalRates[Bar].open,GlobalRates[Bar].close))*pips);
   return(res);
  }
//    

int CCandle::BottomShadowSize(int Bar)
  {
   int res=0;
   res=int((MathMin(GlobalRates[Bar].open,GlobalRates[Bar].close)-GlobalRates[Bar].low)*pips);
   return(res);
  }
//   

uchar CCandle::Type(int Bar)
  {
   uchar res;
   double Open,Close;

   Open=GlobalRates[Bar].open;
   Close=GlobalRates[Bar].close;

   if((Open==Close) || 
      (MathAbs(Open-Close)*pips<=GDoji)) res=DOJICANDLE;
   else
     {
      if(Open>Close) res=BEARCANDLE;
      else                               res=BULLCANDLE;
     }
   return(res);
  }
//+------------------------------------------------------------------+
//|                                      |
//+------------------------------------------------------------------+

// true,      

bool CCandle::LongCandle(int Bar,double deviation=1)
  {
   if(deviation>=0)
     {
      int BodySizeBar=0,TemplateSize;
      if(!MidlleCandle(Bar)) return((Custom.InRange(Codes[Bar],112,127) || Custom.InRange(Codes[Bar],48,63)));
      else
        {
         BodySizeBar=BodySize(Bar);
         if(BodySizeBar>=tplSizes[MIDLLE_CANDLE_BODY])
           {
            if(deviation!=0)
              {
               TemplateSize=tplSizes[LONG_CANDLE_BODY];
               return(BodySizeBar>=MathCeil(TemplateSize*deviation));
              }
            else return(true);
           }
        }
     }
   return(NULL);
  }
// true,     

bool CCandle::MidlleCandle(int Bar)
  {
   return((Custom.InRange(Codes[Bar],96,111) || Custom.InRange(Codes[Bar],32,47)));
  }
// true,       

bool CCandle::SmallCandle(int Bar)
  {
   return((Custom.InRange(Codes[Bar],80,95) || Custom.InRange(Codes[Bar],16,31)));
  }
// true,    

bool CCandle::Doji(int Bar)
  {
   return(Type(Bar)==DOJICANDLE);
  }
// true,     

bool CCandle::BearBar(int Bar)
  {
   return(Type(Bar)==BEARCANDLE);
  }
// true,     

bool CCandle::BullBar(int Bar)
  {
   return(Type(Bar)==BULLCANDLE);
  }
//-( )

bool CCandle::DragonDojiCandle(int Bar)
  {
   return((Codes[Bar]==65) || 
          (Codes[Bar]==66) ||
          (Codes[Bar]==67));
  }
//-

bool CCandle::GraveStoneDojiCandle(int Bar)
  {
   return((Codes[Bar]==68) || 
          (Codes[Bar]==72) ||
          (Codes[Bar]==76));
  }
//-()

bool CCandle::LongLegsDojiCandle(int Bar)
  {
   int code,ushd,lshd;

   code=Codes[Bar];
   ushd=TopShadowSize(Bar);
   lshd=BottomShadowSize(Bar);

   if(code==79 || code==78 || code==75 || code==74)
     {
      if(ushd==lshd) return(true);
      else
        {
         if((ushd>lshd) && (lshd>=(int)MathCeil(ushd*Settings.ES_HW))) return(true);
         else
           {
            if((ushd<lshd) && (ushd>=(int)MathCeil(lshd*Settings.ES_HW))) return(true);
           }
        }
     }

   return(false);
  }
//

bool  CCandle::Maribozu(int Bar)
  {
   return(Custom.DataRange(MB,Codes[Bar]));
  }
//  (96,97,100,112,113,116,32,33,36,48,49,52);
//   

ushort CCandle::CheckMaribozu(int Bar)
  {
   switch(Codes[Bar])
     {
      case 112: return(BULL_MARIBOZU);
      case 113: return(BULL_CLOSE_MARIBOZU);
      case 116: return(BULL_OPEN_MARIBOZU);

      case 48:  return(BEAR_MARIBOZU);
      case 49:  return(BEAR_OPEN_MARIBOZU);
      case 52:  return(BEAR_CLOSE_MARIBOZU);
     }
   return(0);
  }
// 

bool CCandle::HighWave(int Bar)
  {
   bool res=Custom.DataRange(HV,Codes[Bar]);
   int body,ushd,lshd;

   if(res)
     {
      res=false;
      body=BodySize(Bar);
      ushd=TopShadowSize(Bar);
      lshd=BottomShadowSize(Bar);
      if((ushd==lshd) && (lshd>=Settings.ES_Body_HW*body)) return(true);
      else
        {
         if((lshd>ushd) && (ushd>=(int)MathCeil(lshd*Settings.ES_HW)) && (lshd>=Settings.ES_Body_HW*body)) return(true);
         else
           {
            if((lshd<ushd) && (lshd>=(int)MathCeil(ushd*Settings.ES_HW)) && (ushd>=Settings.ES_Body_HW*body)) return(true);
           }
        }
     }

   return(res);
  }
// true, (Bar),    

bool CCandle::LongTopShadows(int Bar)
  {
   switch(Type(Bar))
     {
      case BEARCANDLE: return(Custom.DataRange(BearLongTopShadows,Code(Bar)));
      case BULLCANDLE: return(Custom.DataRange(BullLongTopShadows,Code(Bar)));
      default: return(false);
     }
  }
// true, (Bar),    

bool CCandle::LongBottomShadows(int Bar)
  {
   switch(Type(Bar))
     {
      case BEARCANDLE: return(Custom.DataRange(BearLongBottomShadows,Code(Bar)));
      case BULLCANDLE: return(Custom.DataRange(BullLongBottomShadows,Code(Bar)));
      default: return(false);
     }
  }
//    :
//" "," "," -"

bool CCandle::CandleWithLongTopShadow(int Bar,uchar TypeModel)
  {
   int body=0,ushd=0,lshd=0;
   uchar TypeBar=Type(Bar);

   body=BodySize(Bar);
   ushd=TopShadowSize(Bar);
   lshd=BottomShadowSize(Bar);


   if(TypeBar==BEARCANDLE || TypeBar==BULLCANDLE)
     {
      if(LongTopShadows(Bar))
        {
         switch(TypeModel)
           {
            case 1:
               if(lshd!=0)
                 {
                  if(Custom.Compare(ushd,body,Settings.ES_Body,MORE) && Custom.Compare(ushd,lshd,Settings.ES_Shades,MORE)) return(true);
                 }
               else
                 {
                  if(Custom.Compare(ushd,body,Settings.ES_Body,MORE)) return(true);
                 }
               break;

            case 2:
               if(Custom.Compare(ushd,body,Settings.ES_Body_pin,MORE) && Custom.Compare(ushd,lshd,Settings.ES_Shades_pin,MORE))return(true);
               break;
           }
        }
     }
   return(false);
  }
//    :
// "",""," -"   

bool CCandle::CandleWithLongBottomShadow(int Bar,uchar TypeModel)
  {
   int body=0,ushd=0,lshd=0;
   uchar TypeBar=Type(Bar);

   body=BodySize(Bar);
   ushd=TopShadowSize(Bar);
   lshd=BottomShadowSize(Bar);

   if(TypeBar==BEARCANDLE || TypeBar==BULLCANDLE)
     {
      if(LongBottomShadows(Bar))
        {
         switch(TypeModel)
           {
            case 1:
               if(ushd!=0)
                 {
                  if(Custom.Compare(lshd,body,Settings.ES_Body,MORE) && Custom.Compare(lshd,ushd,Settings.ES_Shades,MORE)) return(true);
                 }
               else
                 {
                  if(Custom.Compare(lshd,body,Settings.ES_Body,MORE)) return(true);
                 }
               break;

            case 2:
               if(Custom.Compare(lshd,body,Settings.ES_Body_pin,MORE) && Custom.Compare(lshd,ushd,Settings.ES_Shades_pin,MORE))return(true);
               break;
           }
        }
     }
   return(false);
  }
//

ushort CCandle::HangingManCandle(int Bar)
  {
   ushort id=1000;
   ushort res=0;
   if(CandleWithLongBottomShadow(Bar,1)) res=id;
   return(res);
  }
//

ushort CCandle::HummerCandle(int Bar)
  {
   ushort id=2000;
   ushort res=0;
   if(CandleWithLongBottomShadow(Bar,1)) res=id;
   return(res);
  }
// 

ushort CCandle::InvertedHummerCandle(int Bar)
  {
   ushort id=3000;
   ushort res=0;
   if(CandleWithLongTopShadow(Bar,1)) res=id;
   return(res);
  }
// 

ushort CCandle::ShootingStarCandle(int Bar)
  {
   ushort id=4000;
   ushort res=0;
   if(CandleWithLongTopShadow(Bar,1)) res=id;
   return(res);
  }
//-()

bool CCandle::BearPinBar(int Bar)
  {
   return(CandleWithLongTopShadow(Bar,2));
  }
//-()

bool CCandle::BullPinBar(int Bar)
  {
   return(CandleWithLongBottomShadow(Bar,2));
  }
//+------------------------------------------------------------------+
