//+------------------------------------------------------------------+
//|                                                       Checks.mqh |
//|                                              Copyright 2010, Kos |
//|                                      http://mql5.com/ru/users/kos|
//+------------------------------------------------------------------+
#property copyright "2010,Kos"

#include <JCPE\Var.mqh>
#include <JCPE\Candle.mqh>
//+------------------------------------------------------------------+
//|                                                                  |
//+------------------------------------------------------------------+
class CEngineChecks
  {
 
protected:
   CCandle            *Candle;
   double            TopThresold;
   double            BottomThresold;
   bool              PriceRange(int Bar);
   uchar             MeetingLine(int Bar,uchar Turn);
   uchar             ThrustingLine(int Bar);
   uchar             InNeckLine(int Bar);
   uchar             OnNeckLine(int Bar);
   ushort            DarkCloud(int Bar);
   ushort            PiercingLine(int Bar);
   bool              CheckLowPriceHarami(double PTop, double f_BottomThresold)  {return(PTop<=f_BottomThresold);}
   bool              CheckHighPriceHarami(double PBottom, double f_TopThresold) {return(PBottom>=f_TopThresold);}
   bool              CheckHaramiCross(int Bar)                                {return(Candle.LongLegsDojiCandle(Bar));}
   bool              CheckBody(const int &Body,const int &MaxBody);
   bool              NearCloseForCrows(int Bar);
   bool              NearMidlle(int Bar);
   uchar             TwoCrows(int Bar);
   uchar             ThreeCrows(void);
   uchar             WhiteSoldiers(const int &SB1,const int &SB2,const int &SB3);
   bool              NearCloseForSoldiers(int Bar);
   uchar             Deliberation(const int &SB3,const int &SB4);
   uchar             Advance_Block(const int &SB2,const int &SB3,const int &SB4);

public:
                     ~CEngineChecks(void);
   void              Init(CCandle *CandleObj);
   bool              CheckRange(int Bar,uchar TypeRange,uchar Side);
   bool              CheckGap(int Bar,uchar between,uchar TypeGAP);
   ushort            CheckAbsorption(int Bar,uchar Turn,uchar Model);
   uchar             CheckBeltHold(int Bar,uchar Turn);
   ushort            CheckTweezers(int Bar,uchar Turn);
   uchar             CheckHarami(int Bar);
   ushort            CheckLines(int Bar,uchar Turn);
   ushort            CheckStars(int Bar,uchar Turn);
   ushort            CheckCrows(int Bar);
   ushort            CheckSoldiers(uchar Turn);
   ushort            CheckThreeStars(uchar Turn,bool Dojies);
  };

void CEngineChecks::~CEngineChecks(void)
   {
   
   }
   
void CEngineChecks::Init(CCandle *CandleObj)
   {
    Candle=CandleObj;
   }  
//+------------------------------------------------------------------+
//|Side=LEFT_SIDE: True, (Bar+1)         |
//|(Bar) - (    )         |
//|                                                                  |
//|Side=RIGHT_SIDE: True, (Bar)          |
//|(Bar+1) - (   )               |
//|------------------------------------------------------------------|
//| :                                                       |
//|  -TypeRange-                                        |
//|   1-  High-Low  AbsorbingBar.  |
//|      AbsorbedBar                             |
//|                                                                  |
//|   2-  Open-Close  AbsorbingBar.|
//|      AbsorbedBar                             |
//|                                                                  |
//|   3-  High-Low  AbsorbingBar.C   |
//|      AbsorbedBar                             |
//|                                                                  |
//|   4-  Open-Close  AbsorbingBar.C |
//|      AbsorbedBar                             |
//+------------------------------------------------------------------+

bool CEngineChecks::CheckRange(int Bar,uchar TypeRange,uchar Side)
  {
   bool    res=false;
   double  Pmin,Pmax;
   int     AbsorbingBar=0,AbsorbedBar=0;

   switch(Side)
     {
      case LEFT_SIDE:
         AbsorbingBar=Bar+1;
         AbsorbedBar=Bar;
         break;

      case RIGHT_SIDE:
         AbsorbingBar=Bar;
         AbsorbedBar=Bar+1;
         break;
     }

   switch(TypeRange)
     {
      case 1:

         Pmin=Candle.Lo(AbsorbingBar);
         Pmax=Candle.Hi(AbsorbingBar);
         res=(Custom.InRange(MathMax(Candle.Cl(AbsorbedBar),Candle.Op(AbsorbedBar)),Pmin,Pmax) && 
              Custom.InRange(MathMin(Candle.Cl(AbsorbedBar),Candle.Op(AbsorbedBar)),Pmin,Pmax));

         break;

      case 2:

         Pmin=MathMin(Candle.Cl(AbsorbingBar),Candle.Op(AbsorbingBar));
         Pmax=MathMax(Candle.Cl(AbsorbingBar),Candle.Op(AbsorbingBar));
         res=(Custom.InRange(MathMax(Candle.Cl(AbsorbedBar),Candle.Op(AbsorbedBar)),Pmin,Pmax) && 
              Custom.InRange(MathMin(Candle.Cl(AbsorbedBar),Candle.Op(AbsorbedBar)),Pmin,Pmax));
         break;

      case 3:

         Pmin=Candle.Lo(AbsorbingBar);
         Pmax=Candle.Hi(AbsorbingBar);
         res=(Custom.InRange(Candle.Hi(AbsorbedBar),Pmin,Pmax) && Custom.InRange(Candle.Lo(AbsorbedBar),Pmin,Pmax));

         break;

      case 4:

         Pmin=MathMin(Candle.Cl(AbsorbingBar),Candle.Op(AbsorbingBar));
         Pmax=MathMax(Candle.Cl(AbsorbingBar),Candle.Op(AbsorbingBar));
         res=(Custom.InRange(Candle.Hi(AbsorbedBar),Pmin,Pmax) && Custom.InRange(Candle.Lo(AbsorbedBar),Pmin,Pmax));

         break;
     }
   return(res);
  }
//+------------------------------------------------------------------+
//|   (  )                     |
//+------------------------------------------------------------------+
//| :                                                       |
//| -between-             |
//|     :                                        |
//|  -1:    (Bar+1)  (Bar)|                                                      
//|  -2:    (Bar-1)  (Bar) |
//| -TypeGAP-                                           |
//|  -UpGAP:    (Bar)                            |
//|  -DownGAP:    (Bar)                          |
//+------------------------------------------------------------------+

bool CEngineChecks::CheckGap(int Bar,uchar between,uchar TypeGAP)
  {
   bool res=false;
   int CheckingBar;

   if(Bar>1)
     {
      if(between==1) CheckingBar=Bar+1;
      else          CheckingBar=Bar-1;
     }
   else CheckingBar=Bar+1;

   switch(TypeGAP)
     {
      case UP_GAP:

         switch(Candle.Type(CheckingBar))
           {
            case BEARCANDLE:

               switch(Candle.Type(Bar))
                 {
                  case BEARCANDLE:
                     res=((Candle.Cl(CheckingBar)>Candle.Op(Bar)) && ((Candle.Cl(CheckingBar)-Candle.Op(Bar))*pips>=GGap));
                     break;

                  case BULLCANDLE:
                     res=((Candle.Cl(CheckingBar)>Candle.Cl(Bar)) && ((Candle.Cl(CheckingBar)-Candle.Cl(Bar))*pips>=GGap));
                     break;

                  case DOJICANDLE:
                     res=((Candle.Cl(CheckingBar)>Candle.Op(Bar)) && ((Candle.Cl(CheckingBar)-Candle.Op(Bar))*pips>=GGap));
                     break;
                 }

               break;

            case BULLCANDLE:

               switch(Candle.Type(Bar))
                 {
                  case BEARCANDLE:
                     res=((Candle.Op(CheckingBar)>Candle.Op(Bar)) && ((Candle.Op(CheckingBar)-Candle.Op(Bar))*pips>=GGap));
                     break;

                  case BULLCANDLE:
                     res=((Candle.Op(CheckingBar)>Candle.Cl(Bar)) && ((Candle.Op(CheckingBar)-Candle.Cl(Bar))*pips>=GGap));
                     break;

                  case DOJICANDLE:
                     res=((Candle.Op(CheckingBar)>Candle.Op(Bar)) && ((Candle.Op(CheckingBar)-Candle.Op(Bar))*pips>=GGap));
                     break;
                 }

               break;

            case DOJICANDLE:

               switch(Candle.Type(Bar))
                 {
                  case BEARCANDLE:
                     res=((Candle.Op(CheckingBar)>Candle.Op(Bar)) && ((Candle.Op(CheckingBar)-Candle.Op(Bar))*pips>=GGap));
                     break;

                  case BULLCANDLE:
                     res=((Candle.Op(CheckingBar)>Candle.Cl(Bar)) && ((Candle.Op(CheckingBar)-Candle.Cl(Bar))*pips>=GGap));
                     break;

                  case DOJICANDLE:
                     res=((Candle.Op(CheckingBar)>Candle.Op(Bar)) && ((Candle.Op(CheckingBar)-Candle.Op(Bar))*pips>=GGap));
                     break;
                 }

               break;
           }

         break;

      case DOWN_GAP:

         switch(Candle.Type(CheckingBar))
           {
            case BEARCANDLE:

               switch(Candle.Type(Bar))
                 {
                  case BEARCANDLE:
                     res=((Candle.Op(CheckingBar)<Candle.Cl(Bar)) && ((Candle.Cl(Bar)-Candle.Op(CheckingBar))*pips>=GGap));
                     break;

                  case BULLCANDLE:
                     res=((Candle.Op(CheckingBar)<Candle.Op(Bar)) && ((Candle.Op(Bar)-Candle.Op(CheckingBar))*pips>=GGap));
                     break;

                  case DOJICANDLE:
                     res=((Candle.Op(CheckingBar)<Candle.Cl(Bar)) && ((Candle.Cl(Bar)-Candle.Op(CheckingBar))*pips>=GGap));
                     break;
                 }

               break;

            case BULLCANDLE:

               switch(Candle.Type(Bar))
                 {
                  case BEARCANDLE:
                     res=((Candle.Cl(CheckingBar)<Candle.Cl(Bar)) && ((Candle.Cl(Bar)-Candle.Cl(CheckingBar))*pips>=GGap));
                     break;

                  case BULLCANDLE:
                     res=((Candle.Cl(CheckingBar)<Candle.Op(Bar)) && ((Candle.Op(Bar)-Candle.Cl(CheckingBar))*pips>=GGap));
                     break;

                  case DOJICANDLE:
                     res=((Candle.Cl(CheckingBar)<Candle.Cl(Bar)) && ((Candle.Cl(Bar)-Candle.Cl(CheckingBar))*pips>=GGap));
                     break;
                 }

               break;

            case DOJICANDLE:

               switch(Candle.Type(Bar))
                 {
                  case BEARCANDLE:
                     res=((Candle.Cl(CheckingBar)<Candle.Cl(Bar)) && ((Candle.Cl(Bar)-Candle.Cl(CheckingBar))*pips>=GGap));
                     break;

                  case BULLCANDLE:
                     res=((Candle.Cl(CheckingBar)<Candle.Op(Bar)) && ((Candle.Op(Bar)-Candle.Cl(CheckingBar))*pips>=GGap));
                     break;

                  case DOJICANDLE:
                     res=((Candle.Cl(CheckingBar)<Candle.Cl(Bar)) && ((Candle.Cl(Bar)-Candle.Cl(CheckingBar))*pips>=GGap));
                     break;
                 }

               break;
           }

         break;
     }
   return(res);
  }
// (Bar) (Bar+1)

ushort CEngineChecks::CheckAbsorption(int Bar,uchar Turn,uchar Model)
  {
   ushort res=0;

   if(Candle.LongCandle(Bar,Settings.ES_LB))
     {
      switch(Turn)
        {
         case UP_TURN:

            switch(Model)
              {
               case BULL:

                  if(Candle.BullBar(Bar) && !Candle.Doji(Bar+1))
                    {
                     if(CheckRange(Bar,2,RIGHT_SIDE)) res=100;
                    }

                  break;

               case BEAR:

                  if(Candle.BearBar(Bar) && !Candle.Doji(Bar+1))
                    {
                     if(CheckRange(Bar,2,RIGHT_SIDE)) res=200;
                    }

                  break;
              }

            break;

         case DOWN_TURN:

            switch(Model)
              {
               case BULL:

                  if(Candle.BullBar(Bar) && !Candle.Doji(Bar+1))
                    {
                     if(CheckRange(Bar,2,RIGHT_SIDE)) res=200;
                    }
                  break;

               case BEAR:

                  if(Candle.BearBar(Bar) && !Candle.Doji(Bar+1))
                    {
                     if(CheckRange(Bar,2,RIGHT_SIDE)) res=100;
                    }

                  break;
              }

            break;
        }
     }
   return(res);
  }
//   

ushort CEngineChecks::CheckTweezers(int Bar,uchar Turn)
  {
   int HighestBar,LowestBar,i,CountedCandles=CountCandles-1;
   double FirstPoint,HighBar,LowBar,error=GDoji/pips;
   ushort res=0;

   switch(Turn)
     {
      case UP_TURN:

         if(Bar!=1)
           {
            if(Candle.Hi(Bar)>Candle.Hi(Bar-1)+error) HighestBar=Bar;
            else                                      HighestBar=Bar-1;
           }
         else HighestBar=Bar;

         FirstPoint=Candle.Hi(HighestBar);

         for(i=HighestBar+1;i<=CountedCandles;i++)
           {
            HighBar=Candle.Hi(i);
            if((HighBar>=FirstPoint-error) && (HighBar<=FirstPoint+error)) res=TWEEZERS;
            else
              {
               if(HighBar>FirstPoint+error) return(0);
              }
           }

         break;

      case DOWN_TURN:

         if(Bar!=1)
           {
            if(Candle.Lo(Bar)<Candle.Lo(Bar-1)-error) LowestBar=Bar;
            else                                      LowestBar=Bar-1;
           }
         else LowestBar=Bar;

         FirstPoint=Candle.Lo(LowestBar);

         for(i=LowestBar+1;i<=CountedCandles;i++)
           {
            LowBar=Candle.Lo(i);
            if((LowBar>=FirstPoint-error) && (LowBar<=FirstPoint+error)) res=TWEEZERS;
            else
              {
               if(LowBar<FirstPoint-error)return(0);
              }
           }
         break;
     }
   return(res);
  }
//    "  "

uchar CEngineChecks::CheckBeltHold(int Bar,uchar Turn)
  {
   uchar res=0,code;

   code=Candle.Code(Bar);
   switch(Turn)
     {
      case UP_TURN:
         if(Candle.Maribozu(Bar))
           {
            res=(uchar)Candle.CheckMaribozu(Bar);
            if((res==120) || (res==121)) res=1;
            else res=0;
           }
         else
            if((code==50) || (code==51)) res=2;
         break;

      case DOWN_TURN:
         if(Candle.Maribozu(Bar))
           {
            res=(uchar)Candle.CheckMaribozu(Bar);
            if((res==100) || (res==102)) res=1;
            else res=0;
           }
         else
            if((code==124) || (code==120))res=2;
         break;
     }
   return(res);
  }
//+------------------------------------------------------------------+
//|                                                                  |
//+------------------------------------------------------------------+

bool CEngineChecks::PriceRange(int Bar)
  {
   double BMax,BMin;
   BMax=MathMax(Candle.Op(Bar),Candle.Cl(Bar));
   BMin=MathMin(Candle.Op(Bar),Candle.Cl(Bar));
   return(((BMax<TopThresold) && (BMin>BottomThresold)));
  }
//    " "

uchar CEngineChecks::MeetingLine(int Bar,uchar Turn)
  {
   uchar res=0;

   switch(Turn)
     {
      case UP_TURN:
         if(Candle.Op(Bar)>Candle.Cl(Bar+1))
           {
            if((Candle.Cl(Bar)>=Candle.Cl(Bar+1)-G_Error) && (Candle.Cl(Bar)<=Candle.Cl(Bar+1)+G_Error))
              {
               if(Candle.BearBar(1) && (Candle.Cl(1)<=Candle.Mid(Bar+1)))
                 {
                  res=90;
                  if(CheckGap(Bar,2,DOWN_GAP)) res+=100;
                 }
              }
           }
         break;

      case DOWN_TURN:
         if(Candle.Op(Bar)<Candle.Cl(Bar+1))
           {
            if((Candle.Cl(Bar)>=Candle.Cl(Bar+1)-G_Error) && (Candle.Cl(Bar)<=Candle.Cl(Bar+1)+G_Error))
              {
               if(Candle.BullBar(1) && (Candle.Cl(1)>=Candle.Mid(Bar+1)))
                 {
                  res=90;
                  if(CheckGap(Bar,2,UP_GAP)) res+=100;
                 }
              }
           }
         break;
     }
   if(res!=0) res+=CheckBeltHold(Bar,Turn);
   return(res);
  }
//    ""

uchar CEngineChecks::ThrustingLine(int Bar)
  {
   uchar res=0;
   if((Candle.Op(Bar)<=Candle.Cl(Bar+1)+G_Error) && 
      (Candle.Cl(Bar)<Candle.Op(Bar+1)) &&
      (Candle.Cl(Bar)<Candle.Mid(Bar+1)))
     {
      if(Candle.BearBar(1) && (Candle.Cl(1)<=BottomThresold) && (Candle.Cl(Bar)>Candle.Cl(Bar+1)+2*G_Error))
        {
         switch(Bar)
           {
            case 2: res=60; break;
            case 3: if(PriceRange(2)) res=(uchar)(60+Bar);                     break;
            case 4: if((PriceRange(3)) && (PriceRange(2))) res=(uchar)(60+Bar); break;
           }
        }
     }
   return(res);
  }
//    "Ha  "

uchar CEngineChecks::InNeckLine(int Bar)
  {
   uchar res=0;

   if(Candle.Op(Bar)<Candle.Cl(Bar+1))
     {
      if((Candle.Cl(Bar)>=Candle.Cl(Bar+1)-G_Error) &&
         (Candle.Cl(Bar)<=Candle.Cl(Bar+1)+G_Error))
        {
         if(Candle.BearBar(1) && (Candle.Cl(1)<=BottomThresold))
           {
            switch(Bar)
              {
               case 2: res=80;                                                  break;
               case 3: if(PriceRange(2)) res=(uchar)(80+Bar);                  break;
               case 4: if(PriceRange(3) && PriceRange(2)) res=(uchar)(80+Bar); break;
              }
           }
        }
     }
   return(res);
  }
//    "  "

uchar CEngineChecks::OnNeckLine(int Bar)
  {
   uchar res=0;
   if((Candle.Op(Bar)<Candle.Lo(Bar+1)) && (Candle.Cl(Bar)>=Candle.Lo(Bar+1)))
     {
      if(Candle.Cl(Bar)<(Candle.Cl(Bar+1)-G_Error))
        {
         if(Candle.BearBar(1) && (Candle.Cl(1)<=BottomThresold))
           {
            switch(Bar)
              {
               case 2: res=70;                                                 break;
               case 3: if(PriceRange(2)) res=(uchar)(70+Bar);                 break;
               case 4: if(PriceRange(3) && PriceRange(2)) res=(uchar)(70+Bar);break;
              }
           }
        }
     }
   return(res);
  }
//    "  " (confirm)

ushort CEngineChecks::PiercingLine(int Bar)
  {
   ushort res=0;
   if(Candle.Op(Bar)-G_Error<=Candle.Cl(Bar+1) && Candle.Cl(Bar)<Candle.Op(Bar+1))
     {
      switch(Bar)
        {
         case 1:
            if(Candle.Cl(Bar)>=Candle.Mid(Bar+1)) res+=40;
            break;

         case 2:
            if(Candle.Cl(Bar)>=Candle.Mid(Bar+1))
              {
               if((Candle.BullBar(1) && (Candle.Cl(1)>Candle.Cl(Bar))) || (Candle.BearPinBar(1) || Candle.BullPinBar(1) || Candle.Doji(1) || Candle.HighWave(1)))
                 {
                  res+=50;
                  if(CheckGap(Bar,2,UP_GAP)) res+=100;
                 }
               else res=ThrustingLine(Bar);
              }
            else
              {
               if(Candle.BullBar(1) && (Candle.Cl(1)>Candle.Mid(Bar+1)))
                 {
                  res+=30;
                  if(CheckGap(Bar,2,UP_GAP)) res+=100;
                 }
               else res=ThrustingLine(Bar);
              }
            if(res!=0) res+=CheckBeltHold(Bar,DOWN_TURN);
            break;
        }
     }
   return(res);
  }
//    "   "

ushort CEngineChecks::DarkCloud(int Bar)
  {
   ushort res=0;

   if(Candle.Op(Bar)+G_Error>=Candle.Cl(Bar+1) && Candle.Cl(Bar)>Candle.Op(Bar+1))
     {
      switch(Bar)
        {
         case 1:
            if(Candle.Cl(Bar)<=Candle.Mid(Bar+1)) res=40;
            break;

         case 2:
            if(Candle.Cl(Bar)<=Candle.Mid(Bar+1))
              {
               if((Candle.BearBar(1) && (Candle.Cl(1)<Candle.Cl(Bar))) || (Candle.BearPinBar(1) || Candle.BullPinBar(1) || Candle.Doji(1) || Candle.HighWave(1)))
                 {
                  res=50;
                  if(CheckGap(Bar,2,DOWN_GAP)) res+=100;
                 }
              }
            else
              {
               if(Candle.BearBar(1) && (Candle.Cl(1)<Candle.Mid(Bar+1)))
                 {
                  res=30;
                  if(CheckGap(Bar,2,DOWN_GAP))res+=100;
                 }
              }
            if(res!=0) res+=CheckBeltHold(Bar,UP_TURN);
            break;
        }
     }
   return(res);
  }
//   ""

uchar CEngineChecks::CheckHarami(int Bar)
  {
   uchar res=0;
   double Pmin,
   Pmax,
   Close,
   Open,
   Midlle,
   TT,
   BT,
   Pmin1,
   Pmax1,
   Open1,
   Close1;
   double  PartBody;
   int body1,body2;

   if(Candle.LongCandle(Bar+1,Settings.ES_LB))
     {
      body1=Candle.BodySize(Bar);
      body2=Candle.BodySize(Bar+1);
      if(CheckRange(Bar,2,LEFT_SIDE) && body1<=MathCeil(body2*Settings.ES_Harami))
        {
         //Bar+1
         PartBody=MathCeil(Candle.BodySize(Bar+1)/3);
         Open1=Candle.Op(Bar+1);
         Close1=Candle.Cl(Bar+1);
         Pmin1=MathMin(Open1,Close1);
         Pmax1=MathMax(Open1,Close1);
         Midlle=Candle.Mid(Bar+1);
         TT=Pmax1-PartBody*point;           // 1/3 
         BT=Pmin1+PartBody*point;           // 1/3 
                                            //Bar
         Open=Candle.Op(Bar);
         Close=Candle.Cl(Bar);
         Pmin=MathMin(Close,Open);
         Pmax=MathMax(Close,Open);

         if((Custom.InRange(Pmax,BT,TT) || Custom.InRange(Pmin,BT,TT)) && !Candle.Doji(Bar)) res=HARAMI;           // 
         else
           {
            if(!Candle.Doji(Bar))
              {
               if(CheckHighPriceHarami(Pmin,TT)) res=HP_HARAMI;                                                    //   
               else
                 {
                  if(CheckLowPriceHarami(Pmax,BT)) res=LP_HARAMI;                                                  //   
                 }
              }
            else
              {
               if(CheckHaramiCross(Bar)) res=CROSS_HARAMI;                                                         //  
              }
           }
        }
     }
   return(res);
  }
//+------------------------------------------------------------------+
//|    :                    |
//|     "  "                                          |
//|     "   "                                   |
//|     "  "                                                |
//|     "  "                                               |
//|     ""                                                     |
//|     " "                                        |
//+------------------------------------------------------------------+

ushort CEngineChecks::CheckLines(int Bar,uchar Turn)
  {
   ushort res=0,id=0;
   switch(Turn)
     {
      case UP_TURN:

         if(Candle.BullBar(Bar+1))
           {
            if(Candle.BearBar(Bar))
              {
               id=7000;
               // TopThresold:=CValues[OPEN_PRICE,Bar+1];
               // BottomThresold:=CValues[OPEN_PRICE,Bar];
               switch(Bar)
                 {
                  case 1: if(UsesPatterns.UseDarkCloud)res=DarkCloud(Bar); break;

                  case 2:
                     //if(UsesPatterns.UseMeetingLine)res=MeetingLine(Bar,Turn);
                     if(res==0 && UsesPatterns.UseDarkCloud) res=DarkCloud(Bar);
                     break;

                 }
              }
           }
         break;

      case DOWN_TURN:

         if(Candle.BearBar(Bar+1))
           {
            if(Candle.BullBar(Bar))
              {
               id=8000;
               TopThresold=Candle.Op(Bar+1);
               BottomThresold=Candle.Op(Bar);
               switch(Bar)
                 {
                  case 1: /*if(UsesPatterns.UsePiercingLine)res=PiercingLine(Bar); */break;

                  case 2:
                     /*if(UsesPatterns.UseMeetingLine)res=MeetingLine(Bar,Turn);
                     if(res==0)
                       {
                        if(UsesPatterns.UsePiercingLine)res=PiercingLine(Bar);
                        if(res==0)
                          {
                           if(UsesPatterns.UseInNeckLine) res=InNeckLine(Bar);
                           if(res==0 && UsesPatterns.UseOnNeckLine) res=OnNeckLine(Bar);
                          }
                       }*/
                     break;

                  case 3:
                     /*if(UsesPatterns.UseThrustingLine)res+=ThrustingLine(Bar);
                     if(res==0 && UsesPatterns.UseInNeckLine) res+=InNeckLine(Bar);
                     if(res==0 && UsesPatterns.UseOnNeckLine) res+=OnNeckLine(Bar);*/
                     break;

                  case 4:
                     /*if(UsesPatterns.UseThrustingLine)res+=ThrustingLine(Bar);
                     if(res==0 && UsesPatterns.UseInNeckLine) res+=InNeckLine(Bar);
                     if(res==0  && UsesPatterns.UseOnNeckLine) res+=OnNeckLine(Bar);*/
                     break;
                 }
              }
           }
         break;
     }

   if(res!=0) return(res+id);
   else return(res);

  }
//+------------------------------------------------------------------+
//|    :                      |
//|"/ "," "                   |
//+------------------------------------------------------------------+
//|   :                                                     |
//|   -Turn    UpTurn,DownTurn       |
//|                           |
//+------------------------------------------------------------------+

ushort CEngineChecks::CheckStars(int Bar,uchar Turn)
  {
   ushort res=0;
   double StarPrice;

   if(Candle.SmallCandle(Bar)) res+=100;
   else
     {
      if(Candle.Doji(Bar)) res+=200;
     }

   if(res>0 && Candle.LongCandle(Bar+1,Settings.ES_LB) && Candle.LongCandle(Bar-1,Settings.ES_LB))
     {

      switch(Turn)
        {
         case UP_TURN:
            if(Candle.BullBar(Bar+1) && Candle.BearBar(Bar-1))
              {
               StarPrice=(double)MathMax(Candle.Op(Bar),Candle.Cl(Bar));
               //---Bar+1
               if(CheckGap(Bar,1,DOWN_GAP)) res+=50;
               else
                 {
                  if(Candle.Cl(Bar+1)/*-G_Error*/<=StarPrice) res+=30;
                  else res=0;
                 }
               //---Bar-1
               if(res!=0)
                 {
                  if(CheckGap(Bar,2,DOWN_GAP)) res+=50;
                  else
                    {
                     if(Candle.Op(Bar-1)/*-G_Error*/<=StarPrice) res+=40;
                     else res=0;
                    }
                 }

               break;

              }
            else return(0);

            break;

         case DOWN_TURN:
            if(Candle.BearBar(Bar+1) && Candle.BullBar(Bar-1))
              {
               StarPrice=MathMin(Candle.Op(Bar),Candle.Cl(Bar));
               //Bar+1
               if(CheckGap(Bar,1,UP_GAP))res+=50;
               else
                 {
                  if(Candle.Cl(Bar+1)/*+G_Error*/>=StarPrice) res+=30;
                  else res=0;
                 }
               //-----

               //Bar-1
               if(res!=0)
                 {
                  if(CheckGap(Bar,2,UP_GAP)) res+=50;
                  else
                    {
                     if(Candle.Op(Bar-1)/*+G_Error*/>=StarPrice) res+=40;
                     else res=0;
                    }
                 }
               //-----
               break;

              }
            else return(0);
            break;
        }
     }
   else return(0);

   return(res);
  }
//   (70), (71)

uchar CEngineChecks::TwoCrows(int Bar)
  {
   uchar res=NULL;

   if(Candle.BearBar(Bar) && Candle.BearBar(Bar+1) && Candle.BullBar(Bar+2))
     {
      if(CheckGap(Bar+1,1,DOWN_GAP))
        {
         if(CheckRange(Bar,2,RIGHT_SIDE))
           {
            res=(Candle.Cl(Bar)>=Candle.Cl(Bar+2)-G_Error)?(uchar)70:(uchar)71;
           }
         else
           {
            res=((Candle.Cl(Bar)<Candle.Cl(Bar+2)) && Custom.InRange(Candle.Op(Bar),Candle.Cl(Bar+1),Candle.Op(Bar+1)))?(uchar)71:(uchar)0;
           }
        }
     }
   return(res);
  }
//   (72),    (73)

uchar CEngineChecks::ThreeCrows(void)
  {
   uchar res=NULL;
   int   SB1,SB2,SB3,MaxBody;

   if(Candle.BearBar(1) && Candle.BearBar(2) && Candle.BearBar(3))
     {
      if(Candle.LongCandle(1,Settings.ES_LB_Crows) && Candle.LongCandle(2,Settings.ES_LB_Crows) && Candle.LongCandle(3,Settings.ES_LB_Crows))
        {
         SB1=Candle.BodySize(1);
         SB2=Candle.BodySize(2);
         SB3=Candle.BodySize(3);
         MaxBody=Custom.GetMax(SB1,SB2,SB3);
         if(CheckBody(SB1,MaxBody) && CheckBody(SB2,MaxBody) && CheckBody(SB3,MaxBody))
           {
            if(NearCloseForCrows(1) && NearCloseForCrows(2)) res+=73;
            else
              {
               if(NearMidlle(1) && NearMidlle(2)) res+=72;
              }
           }
        }
     }
   return(res);
  }
//+------------------------------------------------------------------+
//|                                                                  |
//+------------------------------------------------------------------+
bool   CEngineChecks::CheckBody(const int &Body,const int &MaxBody)
  {
   if(Custom.Compare(Body,MaxBody,Settings.ES_Crows,MORE)) return(true);
   else return(false);
  }
//+------------------------------------------------------------------+
//|                                                                  |
//+------------------------------------------------------------------+
bool   CEngineChecks::NearCloseForCrows(int Bar)
  {
   if(Custom.InRange(Candle.Op(Bar),Candle.Lo(Bar+1)-G_Error,Candle.Cl(Bar+1)+G_Error)) return(true);
   else return(false);
  }
//+------------------------------------------------------------------+
//|                                                                  |
//+------------------------------------------------------------------+
bool   CEngineChecks::NearMidlle(int Bar)
  {
   if(Custom.InRange(Candle.Op(Bar),Candle.Cl(Bar+1)-G_Error,Candle.Mid(Bar+1)+G_Error)) return(true);
   else return(false);
  }
//+------------------------------------------------------------------+
//|    :                    |
//|     "  "                                      |
//|     " "                                                 |
//|     "  "                                          |
//|     "  "                                      |
//+------------------------------------------------------------------+

ushort  CEngineChecks::CheckCrows(int Bar)
  {
   ushort res=0;
   switch(Bar)
     {
      case 1:
         res=ushort(TwoCrows(Bar));
         break;
      case 3:
         res=ushort(ThreeCrows());
         break;
      default: break;
     }

   if(res!=0) return(res+=7000);
   else return(0);
  }
//+------------------------------------------------------------------+
//|                                                   |
//+------------------------------------------------------------------+
uchar  CEngineChecks::WhiteSoldiers(const int &SB1,const int &SB2,const int &SB3)
  {
   uchar res=NULL;
   int   MaxBody=NULL;
   MaxBody=Custom.GetMax(SB1,SB2,SB3);
   if(CheckBody(SB1,MaxBody) && CheckBody(SB2,MaxBody) && CheckBody(SB3,MaxBody))
     {
      if(NearCloseForSoldiers(1) && NearCloseForSoldiers(2)) res+=72;
     }
   return(res);
  }
//+------------------------------------------------------------------+
//|                                                                  |
//+------------------------------------------------------------------+
bool CEngineChecks::NearCloseForSoldiers(int Bar)
  {
   if(Custom.InRange(Candle.Op(Bar),Candle.Mid(Bar+1)-G_Error,Candle.Hi(Bar+1)+G_Error)) return(true);
   else return(false);
  }
//+------------------------------------------------------------------+
//|                                                       |
//+------------------------------------------------------------------+

uchar CEngineChecks::Deliberation(const int &SB3,const int &SB4)
  {
   uchar res=NULL;
   int   MaxBody=NULL;
   if(Candle.LongCandle(4,Settings.ES_LB) && (Candle.MidlleCandle(3) || Candle.LongCandle(3,Settings.ES_LB)) && Candle.SmallCandle(2))
     {
      if(SB4>SB3)
        {
         MaxBody=(int)MathMax(SB3,SB4);
         if(CheckBody(SB3,MaxBody) && CheckBody(SB4,MaxBody))
           {
            if(NearCloseForSoldiers(3) && Candle.Cl(3)<Candle.Mid(2) && Candle.Cl(1)<Candle.Op(2))
              {
               res+=74;
               if(CheckGap(2,1,DOWN_GAP)) res+=100;
              }
           }
        }
     }
   return(res);
  }
//+------------------------------------------------------------------+
//|                                                      |
//+------------------------------------------------------------------+

uchar CEngineChecks::Advance_Block(const int &SB2,const int &SB3,const int &SB4)
  {
   uchar res=NULL;
   if(Candle.LongCandle(4,Settings.ES_LB))
     {
      if(SB4>SB3 && SB3>SB2 && Candle.Cl(1)<Candle.Op(2))
        {
         if(NearCloseForSoldiers(2) && NearCloseForSoldiers(3)) res+=75;
        }
     }
   return(res);
  }
//+------------------------------------------------------------------+
//|    :                    |
//|     "  "                                          |
//|     ""                                                |
//|     " "                                              |
//+------------------------------------------------------------------+

ushort CEngineChecks::CheckSoldiers(uchar Turn)
  {
   ushort res=NULL;
   int SB1,SB2,SB3,SB4,id=0;
   uchar CheckArray[16]={88,89,90,91,92,93,94,95,104,105,106,107,108,109,110,111};

   switch(Turn)
     {
      case UP_TURN:
         id=7000;
         if(Candle.BearBar(1) && Candle.BullBar(2) && Candle.BullBar(3) && Candle.BullBar(4))
           {
            SB2=Candle.BodySize(2);
            SB3=Candle.BodySize(3);
            SB4=Candle.BodySize(4);
            if(UsesPatterns.UseAdvanceBlock)
              {
               if(Custom.DataRange(CheckArray,Candle.Code(3)) && Custom.DataRange(CheckArray,Candle.Code(2))) res+=Advance_Block(SB2,SB3,SB4);
              }

            /*if(UsesPatterns.UseDeliberation && res==0)
              {
               res+=Deliberation(SB3,SB4);
              }*/

            if(res!=0) res+=CheckTweezers(2,Turn);
           }
         break;

      case DOWN_TURN:
         id=8000;
         if(Candle.BullBar(1) && Candle.BullBar(2) && Candle.BullBar(3))
           {
            if(Candle.LongCandle(1,Settings.ES_LB_Crows) && Candle.LongCandle(2,Settings.ES_LB_Crows) && Candle.LongCandle(3,Settings.ES_LB_Crows))
              {
               SB1=Candle.BodySize(1);
               SB2=Candle.BodySize(2);
               SB3=Candle.BodySize(3);
               res+=WhiteSoldiers(SB1,SB2,SB3);
               if(res!=0) res+=CheckTweezers(3,Turn);
              }
           }
         break;
     }
   if(res!=0) res+=(ushort)id;
   return(res);
  }
//+------------------------------------------------------------------+
//|    :                    |
//|     "   " , " "                           |
//+------------------------------------------------------------------+

ushort CEngineChecks::CheckThreeStars(uchar Turn,bool Dojies)
  {
   ushort res=NULL,id=NULL;
   double L1,L2,L3;

   switch(Turn)
     {
      case UP_TURN:
        {
         id=5000;
         res=20;
         res+=CheckGap(2,1,DOWN_GAP)?(ushort)100:(ushort)0;
         res+=CheckGap(2,2,DOWN_GAP)?(ushort)100:(ushort)0;
         if(res!=0)
           {
            res+=CheckTweezers(3,Turn);
            res+=id;
           }
        }
      break;

      case DOWN_TURN:
        {
         id=6000;
         if(!Dojies)
           {
            L1=Candle.Lo(1);
            L2=Candle.Lo(2);
            L3=Candle.Lo(3);
            if(L3==Custom.GetMin(L1,L2,L3) && L2<L1)
              {
               if(CheckRange(1,3,LEFT_SIDE) && CheckRange(2,3,LEFT_SIDE)) res+=10;
               if(res!=0)
                 {
                  res+=CheckTweezers(3,Turn);
                  res+=id;
                 }
              }

           }
         else
           {
            res+=20;
            res+=CheckGap(2,1,UP_GAP)?(ushort)100:(ushort)0;
            res+=CheckGap(2,2,UP_GAP)?(ushort)100:(ushort)0;
            if(res!=0)
              {
               res+=CheckTweezers(3,Turn);
               res+=id;
              }
           }
        }
      break;

      default:break;
     }
   return(res);
  }
 
//+------------------------------------------------------------------+
