//+------------------------------------------------------------------+
//|                                                       Loader.mqh |
//|                                              Copyright 2010, Kos |
//|                                      http://mql5.com/ru/users/kos|
//+------------------------------------------------------------------+
#property copyright "2010,Kos"

#include <JCPE\Const.mqh>
#include <JCPE\Custom.mqh>
//+------------------------------------------------------------------+
//|                                                                  |
//+------------------------------------------------------------------+

class CLoaderManager
  {
protected:
   CCustom           Custom;
   int               tplSize[9];
   double            Mean;
   double            StdDev;
   string            Symb;
   int               LoadSymbolsFromFile(string &Symbols[]);
   bool              SizesArraySave(string Value);
   int               SymbolsArraySave(string &SymbolsList[]);
   bool              InitBodyElements(ENUM_TIMEFRAMES &Periods[]);
   bool              UpdateDataFile(ENUM_TIMEFRAMES period);
   void              MeanAndStdDev(int &Data[],int Size);
public:
   int               SizesArrayLoad(string sy,ENUM_TIMEFRAMES period,int &Sizes[]);
   bool              DataFilesMonitoring(bool MultiMode,uchar &TotalSymbols,ENUM_TIMEFRAMES &Periods[],string &Symbols[]);
   int               LoadHistory(string symbol,ENUM_TIMEFRAMES period,datetime start_date);
   bool              InitBodyElements(string SymbName,ENUM_TIMEFRAMES TimeFrame,int &Data[],int &ResultCode);
  };
//+------------------------------------------------------------------+
//|                                        |
//+------------------------------------------------------------------+

int CLoaderManager::LoadSymbolsFromFile(string &Symbols[])
  {
   string FName="Symbols.CSV";
   int handle=FileOpen(FName,FILE_ANSI|FILE_READ);

   if(handle==INVALID_HANDLE)
     {
      Print("File:",FName," error "+DoubleToString(GetLastError(),0));
      return(-1);
     }

   string Line;
   int    LinesRead=0,Length=0;

   while(!FileIsEnding(handle))
     {
      Line   = FileReadString(handle);
      Length = StringLen(Line);
      if(Length!=0) LinesRead++;
     }

   if(LinesRead!=0)
     {
      FileSeek(handle,0,SEEK_SET);
      ArrayResize(Symbols,LinesRead);
      int i=0;
      while(!FileIsEnding(handle))
        {
         Line   = FileReadString(handle);
         Length = StringLen(Line);
         if(Length!=0)
           {
            Symbols[i]=Line;
            i++;
           }
        }

     }
   FileClose(handle);
   return(LinesRead);
  }
//+------------------------------------------------------------------+
//|                             |
//+------------------------------------------------------------------+

int CLoaderManager::SizesArrayLoad(string sy,ENUM_TIMEFRAMES period,int &Sizes[])
  {
   string FName="MCP.CSV";

   ArrayInitialize(tplSize,0);

   int handle=FileOpen(FName,FILE_ANSI|FILE_READ),i,Start,iPos;
//----
   if(handle==INVALID_HANDLE)
     {
      Print("File:",FName," error "+DoubleToString(GetLastError(),0));
      return(-1);
     }
   string Line,Cur;
   int    LinesRead=0;
//----
   while(!FileIsEnding(handle))
     {
      Line=FileReadString(handle);
      Start=StringLen(Line);
      Cur=sy+Custom.TF(period)+",";
      if(StringFind(Line,Cur,0)==-1 || Start==0)
        {
         continue;
        }
      else
        {
         //Print(Line);
         Start = StringLen(Cur);
         for(i = 0; i <= 8; i++)
           {
            iPos=StringFind(Line,",",Start);
            Sizes[i]=int(StringToInteger(StringSubstr(Line,Start,iPos-Start)));
            Start=iPos+1;
           }
         if(FileIsLineEnding(handle)) LinesRead++;
         break;
        }

     }

   FileClose(handle);
   return(LinesRead);
  }
//+------------------------------------------------------------------+
//|                             |
//+------------------------------------------------------------------+

bool CLoaderManager::SizesArraySave(string Value)
  {
   string FName="MCP.CSV";

   int handle=FileOpen(FName,FILE_ANSI|FILE_READ|FILE_WRITE);

   if(handle<1)
     {
      Print("File:",FName," error "+DoubleToString(GetLastError(),0));
      return(false);
     }
   else
     {
      FileSeek(handle,0,SEEK_END);
      FileWrite(handle,Value);
      FileClose(handle);
      return(true);
     }
  }
//+------------------------------------------------------------------+
//|      Symbols.csv         |
//+------------------------------------------------------------------+

int CLoaderManager::SymbolsArraySave(string &SymbolsList[])
  {
   string FName="Symbols.CSV";
   int handle=FileOpen(FName,FILE_ANSI|FILE_WRITE),i,Size;

   if(handle==INVALID_HANDLE)
     {
      Print("File:",FName," error "+DoubleToString(GetLastError(),0));
      return(-1);
     }
   else
     {
      Size=ArraySize(SymbolsList);
      for(i=0;i<=Size-1;i++)
        {
         FileSeek(handle,0,SEEK_END);
         FileWrite(handle,SymbolsList[i]);
        }

      FileClose(handle);
      return(Size);
     }
  }
//+------------------------------------------------------------------+
//|                                        |
//+------------------------------------------------------------------+

int CLoaderManager::LoadHistory(string symbol,ENUM_TIMEFRAMES period,datetime start_date)
  {
   datetime first_date=0;
   datetime times[100];

   if(symbol==NULL || symbol=="") symbol=Symbol();
   if(period==PERIOD_CURRENT)     period=Period();

   if(!SymbolInfoInteger(symbol,SYMBOL_SELECT))
     {
      if(GetLastError()==ERR_MARKET_UNKNOWN_SYMBOL) return(-1);
      SymbolSelect(symbol,true);
     }

   SeriesInfoInteger(symbol,period,SERIES_FIRSTDATE,first_date);
   if(first_date>0 && first_date<=start_date) return(1);

   if(MQL5InfoInteger(MQL5_PROGRAM_TYPE)==PROGRAM_INDICATOR && Period()==period && Symbol()==symbol)
      return(-4);

   if(SeriesInfoInteger(symbol,PERIOD_M1,SERIES_TERMINAL_FIRSTDATE,first_date))
     {

      if(first_date>0)
        {

         CopyTime(symbol,period,first_date+PeriodSeconds(period),1,times);

         if(SeriesInfoInteger(symbol,period,SERIES_FIRSTDATE,first_date))
            if(first_date>0 && first_date<=start_date)
              {
               return(2);
              }
        }
     }

   int max_bars=TerminalInfoInteger(TERMINAL_MAXBARS);
   bool IsConnected=Custom.IsConnected();
   if(!IsConnected)return(-6);
   datetime first_server_date=0;
   while(!SeriesInfoInteger(symbol,PERIOD_M1,SERIES_SERVER_FIRSTDATE,first_server_date) && !IsStopped())
      Sleep(5);

   if(first_server_date>start_date) start_date=first_server_date;
   if(first_date>0 && first_date<first_server_date)
      Print("Warning: first server date",first_server_date,"for",symbol,"does not match to first series date",first_date);

   int fail_cnt=0;
   while(!IsStopped())
     {
      while(!SeriesInfoInteger(symbol,period,SERIES_SYNCHRONIZED) && !IsStopped())
         Sleep(5);

      int bars=Bars(symbol,period);
      if(bars>0)
        {
         if(bars>=max_bars) return(-2);
         if(SeriesInfoInteger(symbol,period,SERIES_FIRSTDATE,first_date))
            if(first_date>0 && first_date<=start_date) return(0);
        }
      int copied=CopyTime(symbol,period,bars,100,times);
      if(copied>0)
        {
         if(times[0]<=start_date)  return(0);
         if(bars+copied>=max_bars) return(-2);
         fail_cnt=0;
        }
      else
        {
         fail_cnt++;
         if(fail_cnt>=100) return(-5);
         Sleep(10);
        }
     }
   return(-3);
  }
//+------------------------------------------------------------------+
//|    MCP.csv                        |
//+------------------------------------------------------------------+

bool CLoaderManager::InitBodyElements(ENUM_TIMEFRAMES &Periods[])
  {
   bool result=false;
   int  TimeShift,res1,Size=ArraySize(Periods),Writed=0;
   datetime Cur_Time,Start_Time;
   int Sizes[9];

   for(int i=0;i<=Size-1;i++)
     {
      int res=SizesArrayLoad(Symb,Periods[i],Sizes);

      switch(res)
        {
         case -1:
            Print("File open error!");
            break;

         case  0:
            Print(Symb+Custom.TF(Periods[i]),"- Update data process");
            Cur_Time=TimeCurrent();
            TimeShift=FullWeek*(HystoryBars*PeriodSeconds(Periods[i])/TradeWeek);
            Start_Time=Cur_Time-TimeShift;
            Print("Start hystory load",Symb+Custom.TF(Periods[i]),"from",Start_Time);
            //---
            res1=LoadHistory(Symb,Periods[i],Start_Time);

            switch(res1)
              {
               case -1 : Print(" ",Symb);                                   break;
               case -2 : Print("  ,     ");
               if(UpdateDataFile(Periods[i])) Writed++;
               break;
               case -3 : Print("   ");                    break;
               case -4 : Print("     ");          break;
               case -5 : Print("  ");                              break;
               case -6 : Print("    ");                       break;
               case  0 :
                  Print("  ");
                  if(UpdateDataFile(Periods[i])) Writed++;
                  break;

               case  1 :
                  Print("     ");
                  if(UpdateDataFile(Periods[i])) Writed++;
                  break;

               case  2 :
                  Print("     ");
                  if(UpdateDataFile(Periods[i])) Writed++;
                  break;

               default : Print("   ");
              }
            break;

         case 1:
            //Print(Symb+Custom.TF(Periods[i])+" - Loading data complete!");
            Writed++;
            break;
        }
      //if ((res1!=1) && (res1!=2) && (res1!=0)) break; 
     }
   return(Writed==Size);
  }
//+------------------------------------------------------------------+
//|    MCP.csv                        |
//+------------------------------------------------------------------+

bool CLoaderManager::InitBodyElements(string SymbName,ENUM_TIMEFRAMES TimeFrame,int &Data[],int &ResultCode)
  {
   bool result=false;
   int res=SizesArrayLoad(SymbName,TimeFrame,Data);
   ResultCode=res;
   Symb=SymbName;
   switch(res)
     {
      case -1:
      case  0:
        {
         if(res<0) Custom.FileCreate("MCP.CSV");
         int  TimeShift,res1;
         datetime Cur_Time,Start_Time;
         Print(SymbName+Custom.TF(TimeFrame),"- Update data process");
         Cur_Time=TimeCurrent();
         TimeShift=FullWeek*(HystoryBars*PeriodSeconds(TimeFrame)/TradeWeek);
         Start_Time=Cur_Time-TimeShift;
         Print("Start hystory load ",SymbName+Custom.TF(TimeFrame)," from ",Start_Time);
         //---
         res1=LoadHistory(SymbName,TimeFrame,Start_Time);

         switch(res1)
           {
            case -1 : Print("Unknow symbol",Symb);                             break;
            case -2 :
              {
               Print("Requested bars more than max bars in chart ");
               if(UpdateDataFile(TimeFrame)) result=true;
              }
            break;
            case -3 : Print("Program was stopped by user");                    break;
            case -4 : Print("Indicator shouldn't load its own data");          break;
            case -5 : Print("Load failed");                                    break;
            case -6 : Print("No connection");                                  break;
            case  0 :
              {
               Print("Loaded OK");
               if(UpdateDataFile(TimeFrame)) result=true;
              }
            break;

            case  1 :
              {
               Print("Loaded previously");
               if(UpdateDataFile(TimeFrame)) result=true;
              }
            break;

            case  2 :
              {
               Print("Loaded previously and built");
               if(UpdateDataFile(TimeFrame)) result=true;
              }
            break;

            default : Print("Unknown result");
           }
        }
      break;

      case 1:
         return(true);
         break;
     }

   return(result);
  }
//+------------------------------------------------------------------+
//|                                        |
//+------------------------------------------------------------------+

bool CLoaderManager::UpdateDataFile(ENUM_TIMEFRAMES period)
  {
   int    bars;
   bool   synchronized=false;
   int    attempts=0;
   int    body[],ushd[],lshd[];
   ushort digit=ushort(SymbolInfoInteger(Symb,SYMBOL_DIGITS));
   int    pip=int(MathPow(10,digit));
   string SValue;

   MqlRates rates[];
   bars=Bars(Symb,period);
   int copied=CopyRates(Symb,period,0,bars,rates);
   if(copied>0)
     {
      Print("Copied =",copied);
      ArrayResize(body,copied);
      ArrayResize(ushd,copied);
      ArrayResize(lshd,copied);
      for(int i=0; i<=copied-1;i++)
        {
         body[i]=int(MathAbs(rates[i].open-rates[i].close)*pip);

         lshd[i]=int((MathMin(rates[i].open,rates[i].close)-rates[i].low)*pip);

         ushd[i]=int((rates[i].high-MathMax(rates[i].open,rates[i].close))*pip);
        }
      // Print("Ok")         ; 
      MeanAndStdDev(body,copied);
      Print("Ok!!!");
      tplSize[LONG_CANDLE_BODY]     =int(MathCeil(Mean+0.5*StdDev));                                         //   
      tplSize[MIDLLE_CANDLE_BODY]   =int(Mean);                                                              //   
      tplSize[SHORT_CANDLE_BODY]    =int(MathCeil(Mean-0.5*StdDev));                                         //   

      MeanAndStdDev(lshd,copied);

      tplSize[LONG_BOTTOM_SHADOW]   =int(MathCeil(Mean+0.5*StdDev));                                         //     
      tplSize[MIDLLE_BOTTOM_SHADOW] =int(Mean);                                                              //     
      tplSize[SHORT_BOTTOM_SHADOW]  =int(MathCeil(Mean-0.5*StdDev));                                         //     

      MeanAndStdDev(ushd,copied);

      tplSize[LONG_TOP_SHADOW]      =int(MathCeil(Mean+0.5*StdDev));                                         //     
      tplSize[MIDLLE_TOP_SHADOW]    =int(Mean);                                                              //      
      tplSize[SHORT_TOP_SHADOW]     =int(MathCeil(Mean-0.5*StdDev));                                         //     


      StringConcatenate(SValue,Symb,Custom.TF(period),",",tplSize[SHORT_CANDLE_BODY],",",tplSize[LONG_CANDLE_BODY],
                        ",",tplSize[MIDLLE_CANDLE_BODY],
                        ",",tplSize[SHORT_TOP_SHADOW],",",tplSize[LONG_TOP_SHADOW],
                        ",",tplSize[MIDLLE_TOP_SHADOW],
                        ",",tplSize[SHORT_BOTTOM_SHADOW],",",tplSize[LONG_BOTTOM_SHADOW],
                        ",",tplSize[MIDLLE_BOTTOM_SHADOW]);

      return(SizesArraySave(SValue));
     }
   else
     {
      Print("Loading history failed ",Symb+Custom.TF(period));
      return(false);
     }

   return(synchronized);
  }
//+------------------------------------------------------------------+
//|                     |
//+------------------------------------------------------------------+

void CLoaderManager::MeanAndStdDev(int &Data[],int Size)
  {
   double Sum=0,S=0;
   int i;
   for(i=0;i<=Size-1;i++)
     {
      Sum+=Data[i];
     }
   Mean=MathCeil(Sum/Size);

   for(i=0;i<=Size-1;i++)
     {
      S+=MathPow((Mean-Data[i]),2);
     }
   StdDev=MathCeil(MathSqrt(S/Size-1));
  }
//+------------------------------------------------------------------+
//|                             |
//+------------------------------------------------------------------+

bool CLoaderManager::DataFilesMonitoring(bool MultiMode,uchar &TotalSymbols,ENUM_TIMEFRAMES &Periods[],string &Symbols[])
  {
   bool result=NULL;
   int i;
   uchar LoadingCount=NULL;
   uchar SymbolsCount=NULL;

   if(MultiMode)
     {
      if(Custom.CheckFile("Symbols.CSV")==-1) Custom.FileCreate("Symbols.CSV");

      SymbolsCount=uchar(LoadSymbolsFromFile(Symbols));
      if(SymbolsCount==0)
        {
         SymbolsCount=(uchar)SymbolsTotal(true);
         if(SymbolsCount>0)
           {
            ArrayResize(Symbols,SymbolsCount);
            for(i=0;i<=SymbolsCount-1;i++)
              {
               Symbols[i]=SymbolName(i,true);
              }
            SymbolsCount=(uchar)SymbolsArraySave(Symbols);
            Print("SymbolsList count: ",SymbolsCount);
           }
        }
     }
   else
     {
      SymbolsCount=1;
      ArrayResize(Symbols,SymbolsCount);
      Symbols[0]=Symbol();
     }

   if(SymbolsCount>0)
     {
      if(Custom.CheckFile("MCP.CSV")==-1) Custom.FileCreate("MCP.CSV");

      for(i=0;i<=SymbolsCount-1;i++)
        {
         Symb=Symbols[i];
         if(InitBodyElements(Periods))
           {
            Print("Checking data for "+Symb+" complete!");
            LoadingCount++;
           }
         else
           {
            Print("Checking data for "+Symb+" failed!");
            break;
           }
        }

      if(LoadingCount==SymbolsCount)
        {
         Print("Initializing data complete!");
         TotalSymbols=SymbolsCount;
         return(true);
        }
      else
        {
         Print("Initializing data failed!");
         return(NULL);
        }
     }
   return(result);
  }
//+------------------------------------------------------------------+
